@@ -21,8 +21,9 @@
 #' }
 #'
 #' @param inputId The input slot that will be used to access the value
+#' @param label label for input field
 #' @param inputClass class of the matrix input html element
-#' @param value Inital value. Should be a matrix
+#' @param value Initial value. Should be a matrix
 #' @param class Matrix will be coerced to a matrix of this class. `character` and `numeric`
 #' are supported
 #' @param rows list of options to configure rows
@@ -43,6 +44,7 @@
 #'
 #' @export
 matrixInput <- function(inputId,
+                        label = NULL,
                         value = matrix("", 1, 1),
                         inputClass = "",
                         rows = list(),
@@ -52,21 +54,28 @@ matrixInput <- function(inputId,
                         copy = FALSE,
                         copyDoubleClick = FALSE){
   stopifnot(is.matrix(value))
+
+  inputField <- tags$div(
+    id = inputId,
+    class = paste("matrix-input", inputClass),
+    "data-data" = jsonlite::toJSON(value),
+    "data-rownames" = jsonlite::toJSON(rownames(value)),
+    "data-colnames" = jsonlite::toJSON(colnames(value)),
+    "data-rows" = jsonlite::toJSON(rows, auto_unbox = TRUE),
+    "data-cols" = jsonlite::toJSON(cols, auto_unbox = TRUE),
+    "data-class" = class,
+    "data-copy" = jsonlite::toJSON(copy, auto_unbox = TRUE),
+    "data-paste" = jsonlite::toJSON(paste, auto_unbox = TRUE),
+    "data-copyDoubleClick" = jsonlite::toJSON(copyDoubleClick, auto_unbox = TRUE)
+  )
+
   tagList(
     singleton(tags$head(tags$script(src = "shinyMatrix/matrixInput.js"))),
     singleton(tags$head(tags$link(rel = "stylesheet", type = "text/css", href = "shinyMatrix/matrixInput.css"))),
-    tags$div(
-      id = inputId,
-      class = paste("matrix-input", inputClass),
-      "data-data" = jsonlite::toJSON(value),
-      "data-rownames" = jsonlite::toJSON(rownames(value)),
-      "data-colnames" = jsonlite::toJSON(colnames(value)),
-      "data-rows" = jsonlite::toJSON(rows, auto_unbox = TRUE),
-      "data-cols" = jsonlite::toJSON(cols, auto_unbox = TRUE),
-      "data-class" = class,
-      "data-copy" = jsonlite::toJSON(copy, auto_unbox = TRUE),
-      "data-paste" = jsonlite::toJSON(paste, auto_unbox = TRUE),
-      "data-copyDoubleClick" = jsonlite::toJSON(copyDoubleClick, auto_unbox = TRUE)
+    div(
+      class = "form-group shiny-matrix-input-container shiny-input-container",
+      if (!is.null(label)) tags$label(label, `for` = inputId) else NULL,
+      inputField
     )
   )
 }
